"""Type stubs for the _pretty_mod Rust extension module."""

from typing import Any

class ModuleTreeExplorer:
    def __init__(self, root_module_path: str, max_depth: int = 2) -> None: ...
    @property
    def root_module_path(self) -> str: ...
    @property
    def max_depth(self) -> int: ...
    @property
    def tree(self) -> dict[str, Any]: ...
    def explore(self) -> dict[str, Any]: ...
    def get_tree_string(self) -> str: ...

def display_tree(root_module_path: str, max_depth: int = 2) -> None: ...
def display_signature(import_path: str) -> str: ...
def import_object(import_path: str) -> Any: ...
